<?php

declare(strict_types=1);

namespace App\Controller;

use App\Aggregate\AcceptOrderCommand;
use App\Contract\Order;
use App\Exception\ClientLockedException;
use App\Exception\ClientNotExistsException;
use App\Exception\OrderInvalidException;
use App\Service\ClientService;
use App\Service\OrderService;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\HttpKernel\Attribute\MapRequestPayload;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Serializer\SerializerInterface;

#[AsController]
#[Route(path: '/orders', name: 'orders_')]
readonly class OrderController
{
    public function __construct(
        private ClientService $clientService,
        private OrderService $orderService,
        private SerializerInterface $serializer,
    ) {

    }

    #[Route(name: 'accept', methods: ['POST'])]
    public function accept(#[MapRequestPayload] Order $orderDto): Response
    {
        try {
            $command = new AcceptOrderCommand(
                order: $orderDto,
                client: $this->clientService->getClient($orderDto->getClientId()),
            );

            $this->orderService->acceptOrder($command);

            return new Response();
        } catch (ClientNotExistsException) {
            return new Response(status: Response::HTTP_UNPROCESSABLE_ENTITY);
        } catch (ClientLockedException) {
            return new Response(status: Response::HTTP_FORBIDDEN);
        } catch (OrderInvalidException $orderInvalidException) {
            return new JsonResponse(
                $this->serializer->serialize($orderInvalidException->getViolations(), format: 'json'),
                status: Response::HTTP_BAD_REQUEST,
                json: true
            );
        }
    }

}
