<?php

namespace App\Entity;

use App\Repository\ClientRepository;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\Types\UuidType;
use Symfony\Component\Uid\Uuid;

#[ORM\Entity(repositoryClass: ClientRepository::class)]
class Client
{
    public function __construct(
        #[ORM\Id]
        #[ORM\Column(type: UuidType::NAME, unique: true)]
        private ?Uuid $clientId = null,

        #[ORM\Column]
        private ?string $name = null,

        #[ORM\Column]
        private ?int $initialBalance = null,

        #[ORM\Column]
        private ?int $currentBalance = null,

        #[ORM\Column(nullable: true)]
        private ?\DateTimeImmutable $lockedAt = null,
    ) {
        $this->currentBalance ??= $this->initialBalance;
    }

    public function getClientId(): ?Uuid
    {
        return $this->clientId;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): void
    {
        $this->name = $name;
    }

    public function getInitialBalance(): ?int
    {
        return $this->initialBalance;
    }

    public function setInitialBalance(?int $initialBalance): void
    {
        $this->initialBalance = $initialBalance;
    }

    public function getBalance(): ?int
    {
        return $this->currentBalance;
    }

    public function deduceFromBalance(int $amount): void
    {
        $this->currentBalance -= $amount;
    }

    public function topUpBalance(int $amount): void
    {
        $this->currentBalance += $amount;
    }

    public function setLockedAt(?\DateTimeImmutable $dateTime)
    {
        $this->lockedAt = $dateTime;
    }

    public function getLockedAt(): ?\DateTimeImmutable
    {
        return $this->lockedAt;
    }

    public function isLocked(): bool
    {
        return $this->lockedAt !== null;
    }
}
